% Option Valuation by Implicit Finite Differences
% Explanation taken from Clewlow and Strickland's book
% "Implementing Derivatives Models", Section 3.6.

clc; clear;

% Option features.
T = 0.5;          % Maturity
Spot = 100;       % Spot price
K = 120;          % Strike price
v = 0.20;         % Volatility
r = 0.06;         % Risk free rate
q = 0.03 ;        % Dividend yield
PutCall = 'P';    % 'C'all or 'P'ut
EuroAmer = 'A';   % 'E'uropean or 'A'merican

% PDE features.
N = 1000  ;            % Number of time steps
M = 200  ;             % Number of stock price steps
dt = T/N;              % Time increment
mu = r - q - v^2/2;    % Drift for stock process
dx = v*sqrt(3*dt);     % Increment for stock price

% Probabilities.
pu = -dt/2*(v^2/dx^2 + mu/dx);        % Up probability
pm = 1 + dt*v^2/dx^2 + r*dt;          % Middle probability
pd = -dt/2*(v^2/dx^2 - mu/dx);        % Down probability

% Initialize stock price and option value.
S = zeros(2*M+1, N+1);
V = zeros(2*M+1, N+1);

% Indices for stock price step.
J = [M:-1:-M]';

% Stock price at maturity
S = Spot*exp(J.*dx);
clear J

% Option price at maturity is intrinsic value.
if strcmp(PutCall,'P')
	V(:,end) = max(K - S, 0);
else
	V(:,end) = max(S - K, 0);
end

% Upper and lower boundaries for the grid.
if strcmp(PutCall,'P')
	lambda_L = max(0,K - S(2*M+1));
	lambda_U = 0;
elseif strcmp(PutCall,'C')
	lambda_L = 0;
	lambda_U = max(0,S(1) - K);
end

for j=N:-1:1
	% Create the 'C' vector.
	C = [lambda_U; V(2:2*M,j+1); lambda_L];
	% Solve the triangular system of equations.
	V(:,j) = SolveTriangular(C,pu,pm,pd,lambda_L,lambda_U);
	% Apply early exercise to American options.
	if strcmp(EuroAmer,'A')
		for i=1:2*M+1;
			switch PutCall
				case 'P'
					V(i,j) = max(V(i,j), K - S(i));   % Puts.
				case 'C'
					V(i,j) = max(V(i,j), S(i) - K);   % Calls.
			end
		end
	end
end
clear i j

% Output the price and compare to the trinomial tree.
ImplicitPrice = V(M+1,1)
TrinTreePrice = Trinomial_Tree(Spot,K,r,q,v,T,M,EuroAmer,PutCall)

